
using StructuresKit

member_definitions = [(1.0, 1.0/12, 1, 1)]

z, m = BeamColumn.discretize(member_definitions)

#A Ix Iy J Cw xc yc xs ys
section_properties = [(1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0)]

material_properties = [(1.0, 0.3)]

num_nodes = length(z)

kx = zeros(Float64, num_nodes)
ky = zeros(Float64, num_nodes)
kϕ = zeros(Float64, num_nodes)
hx = zeros(Float64, num_nodes)
hy = zeros(Float64, num_nodes)

qy = ones(Float64, num_nodes)
qx = zeros(Float64, num_nodes)

ax = zeros(Float64, num_nodes)
ay = zeros(Float64, num_nodes)

E = material_properties[1][1]
Ix = section_properties[1][2]
L = z[end]
Pcr = π^2*E*Ix/L^2

P = ones(Float64, num_nodes) .* Pcr * 0.8

#end boundary conditions
#type=1 u''=v''=ϕ''=0 (simply supported), type=2 u'=v'=ϕ'=0  (fixed), type=3 u''=v''=ϕ''=u'''=v'''=ϕ'''=0 (free end, e.g., a cantilever)
end_boundary_conditions = [1 1]

#supports
#location where u=v=ϕ=0
supports = [0.0 L]

#Set up the free flange model.
model = BeamColumn.define(z, m, member_definitions, section_properties, material_properties, kx, ky, kϕ, hx, hy, qx, qy, P, ax, ay, end_boundary_conditions, supports)

#Run the free flange model.
model = BeamColumn.solve(model)

#Chajes Chapter 3 Beam-Column solution.
δo = 5*qy[1]*L^4/(384 * E * Ix)

δ = δo * 1/(1-P[1]/Pcr)

δ_model = maximum(model.v)

maximum(model.v)

using Plots
plot(model.z, model.v)

