using StructuresKit

#Verify Julia solution against Mathematica solution from
#Example 2 from Plaut, R.H., Moen, C.D.(2020). "Lateral-Torsional Deformations of Single-Span and Two-Span Thin-Walled Beams with Continuous Bracing".
#Journal of Constructional Steel Research.

#Z-section, no slope, single span, fixed-fixed
#kϕ=1500 N*mm/rad/mm, kx=0.1 N/mm^2, gravity load
#q = 1 kN/m
#Check again Figure 14 in the Plaut and Moen (2020) manuscript.
#  max ϕ

plaut_solution = 0.0164

#*********** Julia solution

#inputs

#Ix Iy Ixy J Cw
section_properties = [(3.230E6,449530,-865760, 397.09, 3.4104E9)]

#E  ν
material_properties = [(200,0.30)]


#member information
#L dL SectionProperties MaterialProperties
member_definitions = [(7620.0,7620.0/48,1,1)]

#end boundary conditions
#type=1 u''=v''=ϕ''=0 (simply supported), type=2 u'=v'=ϕ'=0  (fixed), type=3 u''=v''=ϕ''=u'''=v'''=ϕ'''=0 (free end, e.g., a cantilever)
end_boundary_conditions = [2 2]

#supports
supports = [(0.0, "fixed", "fixed", "fixed"), (7620.0, "fixed", "fixed", "fixed")]

supports = [(0.0, "free", "fixed", "fixed"), (7620.0, "fixed", "fixed", "fixed")]

supports = [(0.0, "fixed", "fixed", "free"), (7620.0/2, "fixed", "fixed", "free"), (7620.0, "fixed", "fixed", "free")]


dz, z, m = Mesh.define_line_element(member_definitions)

num_nodes = length(z)

#ax ay
ax = ones(Float64, num_nodes) * 27.826
ay = ones(Float64, num_nodes) * 101.6

#kx kϕ
kx = Array{Array{Float64}}(undef, 1)
kϕ = Array{Array{Float64}}(undef, 1)

kx[1] = ones(Float64, num_nodes) * 0.1/1000
kϕ[1] = ones(Float64, num_nodes) * 1500/1000

#ay_kx
ay_kx = Array{Array{Float64}}(undef, 1)
ay_kx[1] =  ones(Float64, num_nodes) * 101.6

qx = ones(Float64, num_nodes) * 0.001
qy = ones(Float64, num_nodes) * 5.0/1000


model = ThinWalledBeam.define(z, m, member_definitions, section_properties, material_properties, kx, kϕ, ay_kx, qx, qy, ax, ay, end_boundary_conditions, supports)

model = ThinWalledBeam.solve(model)

using Plots
plot(model.z, model.u)
plot(model.z, model.v)
plot(model.z, model.ϕ)

cdm = []

cdm1 = [4]

cdm2 = [cdm1; cdm]

K = rand(4, 4)

free_dof = [1, 3]

K_ff = K[free_dof, free_dof]
